#!/bin/bash
#
# clone.sh <LinuxUserID> - clone a Linux server running under z/VM
#
# For details on how this script works see the book: 
#   "z/VM and Linux on IBM System z: The Virtualization Cookbook"
#   on the Web at: http://linuxvm.org/present/misc/virt-cookbook-1.pdf  
#
# ----------------------------------------------------------------------------
# THE PROGRAM IS PROVIDED ON AN "AS IS" BASIS, WITHOUT WARRANTIES OR CONDITIONS
# OF ANY KIND, EITHER EXPRESS OR IMPLIED INCLUDING, WITHOUT LIMITATION, ANY
# WARRANTIES OR CONDITIONS OF TITLE, NON-INFRINGEMENT, MERCHANTABILITY
# OR FITNESS FOR A PARTICULAR PURPOSE.
# NEITHER RECIPIENT NOR ANY CONTRIBUTORS SHALL HAVE ANY LIABILITY FOR ANY
# DIRECT, INDIRECT, INCIDENTAL, SPECIAL, EXEMPLARY, OR CONSEQUENTIAL DAMAGES
# (INCLUDING WITHOUT LIMITATION LOST PROFITS), HOWEVER CAUSED AND ON ANY THEORY
# OF LIABILITY, WHETHER IN CONTRACT, STRICT LIABILITY, OR TORT (INCLUDING
# NEGLIGENCE OR OTHERWISE) ARISING IN ANY WAY OUT OF THE USE OR
# DISTRIBUTION OF THE PROGRAM OR THE EXERCISE OF ANY RIGHTS GRANTED
# HEREUNDER, EVEN IF ADVISED OF THE POSSIBILITY OF SUCH DAMAGES
# ----------------------------------------------------------------------------
 
#+--------------------------------------------------------------------------+
function help()
# give help 
#+--------------------------------------------------------------------------+
 {
  echo "Usage: clone [-v] target_linux_ID"
  echo ""
  echo "  Clone Linux system from $master_mnt_pt to target_linux_ID"
  echo "  Option -v: verbose"
  exit 1
 }

#+--------------------------------------------------------------------------+
function process_arguments() 
# give help 
#+--------------------------------------------------------------------------+
 {
  verbose="off"
  if [ $# = 0 -o $# -gt 2 ]; then # user did not pass 1 or 2 arguments
    echo "Error: wrong number of arguments"
    echo ""
    help 
  fi
  for arg in $*
  do
    case $arg in
      -v) 
         verbose="on"
         ;;
      *)
        clone_id=`echo $arg | tr '[a-z]' '[A-Z]'`     # fold target user ID to upper case
        ;;
    esac
  done
  if [ $clone_id = "" ]; then # new linux user ID was not passed 
    echo "Error: new Linux user ID must be an argument"
    echo ""
    help 
  fi
 }

#+--------------------------------------------------------------------------+
function cp_cmd()
# echo a CP command and invoke it via the vmcp module/command
#   Arg1-n: the command to issue 
#   Return: the command's return code
#+--------------------------------------------------------------------------+
 {
  echo "Invoking CP command: $@"
# parse output to get return code: awk -F# splits line at '#' with rc at end
  output=`vmcp $@ 2>&1`
  echo "$output"
  ret_val=0
  ret_val=`echo $output | grep "Error: non-zero CP response" | awk -F# '{print $2}'`
  return $ret_val
 }

#+--------------------------------------------------------------------------+
function check_target_id()
# Verify user ID exists and is logged off 
#+--------------------------------------------------------------------------+
 {
  cp_cmd QUERY $clone_id
  case $? in
    0)  # user ID is logged on or disconnected 
      echo "$clone_id user ID must be logged off"
      exit 2
      ;;
    3)  # user ID does not exist
      echo "$clone_id user ID does not exist"
      exit 3
      ;;
   45) # user ID is logged off - this is correct
      ;;
   *) # unexpected
      echo "$clone_id user ID must exist and be logged off"
      exit 4
  esac
 }

#+--------------------------------------------------------------------------+
function prepare_ipaddr()
# Prepare an IP address by adding a backslash before any "."s 
# to make it 4 digits
#    Arg 1: The IP address to be modified
# Return:
#    The new value is written to the global variable new_ipaddr
#+--------------------------------------------------------------------------+
 {
  new_ipaddr=`echo $1 | sed -e 's:\.:\\\.:g'`
 }

#+--------------------------------------------------------------------------+
function prepare_vaddr()
# Prepare a z/VM virtual address by folding to lower case and prepending leading zeros
# to make it 4 digits
#    Arg 1: The vaddr to be modified
# Return:
#    The new value is written to the global variable new_vaddr
#+--------------------------------------------------------------------------+
 {
  new_vaddr=`echo $1 | tr '[A-Z]' '[a-z]'`  # fold to lower case
  let leadingZeros=4-${#1}                  # determine number of zeros to add
  let i=0
  while [ $i -lt $leadingZeros ]; do
    new_vaddr="0$new_vaddr"
    i=$[$i+1]
  done
 }

#+--------------------------------------------------------------------------+
function copy_disk()
# Try to use z/VM FLASHCOPY to copy one disk to another. 
#   Arg 1: Source minidisk vaddr
#   Arg 2: the word "to"
#   Arg 3: Target user ID 
#   Arg 4: Target virtual address
# Return code
#   0: success with FLASHCOPY
#   1: success with dasdfmt and dd
#   2: Target vaddr already in use?
#   3: CP LINK failed
#   4: unable to enable target minidisk 
#   5: can't find source disk in /dev/dasd/devices 
#   6: can't find target disk in /dev/dasd/devices 
#   7: dasdfmt failed
#   8: dd failed
#+--------------------------------------------------------------------------+
 {
  ret_val=0
  source_mdisk=$1
  target_userid=$3
  target_vaddr=$4
  cp_cmd QUERY VIRTUAL $target_vaddr  
  rc=$?
  if [ $rc != 40 ]; then 
    echo "Error: CP QUERY VIRTUAL $target_vaddr failed with $rc"
    return 2 
  fi
  cp_cmd LINK $target_userid $source_mdisk $target_vaddr MR
  rc=$?
  if [ $rc != 0 ]; then # LINK failed
    echo "Error: CP LINK $target_userid $source_mdisk $target_vaddr failed with $rc"
    return 3
  fi
  cp_cmd FLASHCOPY $source_mdisk 0 END $target_vaddr 0 END 
  rc=$?
  if [ $rc != 0 ]; then # FLASHCOPY failed
    echo "FLASHCOPY $source_mdisk $target_vaddr failed with $rc"
    echo "Falling back to dasdfmt and dd copy"
#   enable target disk
    sleep 1
    chccwdev -e 0.0.$target_vaddr
    rc=$?
    if [ $rc != 0 ]; then # unable to enable target disk
      echo "Error: unable to enable 0.0.$target_disk, rc from chccwdev = $rc"
      cp_cmd DETACH $target_vaddr
      return 4
    fi
#   get device name of source disk
    source_dev=`cat /proc/dasd/devices | grep "$source_mdisk(ECKD)" | awk '{ print $7 }'`
    if [ "$source_dev" = "" ]; then
      cat /proc/dasd/devices
      echo "Error: can't find $source_mdisk(ECKD) in /proc/dasd/devices"
      chccwdev -d 0.0.$target_vaddr # clean up
      cp_cmd DETACH $target_vaddr
      return 5
    fi
#   get device name of target disk
    target_dev=`cat /proc/dasd/devices | grep "$target_vaddr(ECKD)" | awk '{ print $7 }'`
    if [ "$target_dev" = "" ]; then
      cat /proc/dasd/devices
      echo "Error: can't find $target_vaddr(ECKD) in /proc/dasd/devices"
      chccwdev -d 0.0.$target_vaddr # clean up
      cp_cmd DETACH $target_vaddr
      return 6 
    fi
#   dasdfmt target disk
    echo "Invoking command: dasdfmt -b 4096 -y -f /dev/$target_dev"
    dasdfmt -b 4096 -y -f /dev/$target_dev
    rc=$?
    if [ $rc != 0 ]; then # dasdfmt failed
      echo "Error: dasdfmt -b 4096 -y -f /dev/$target_dev failed with $rc"
      chccwdev -d 0.0.$target_vaddr # clean up
      cp_cmd DETACH $target_vaddr
      return 7
    fi
    nblks=`cat /proc/dasd/devices | grep $target_dev | awk '{ print $13 }'`
#   copy from source disk to target via dd
    echo "Invoking command: dd bs=4096 count=$nblks if=/dev/$source_dev of=/dev/$target_dev"
    dd bs=4096 count=$nblks if=/dev/$source_dev of=/dev/$target_dev
    rc=$?
    if [ $rc != 0 ]; then # dd failed
      echo "Error: dd bs=4096 count=$nblks if=/dev/$source_dev of=/dev/$target_dev failed with $rc"
      chccwdev -d 0.0.$target_vaddr # clean up
      cp_cmd DETACH $target_vaddr
      return 8
    fi
    chccwdev -d 0.0.$target_vaddr
    echo "Copying disk via dasdfmt/dd succeeded ..."
    ret_val=1 # success with dasdfmt and dd
  fi # if FLASHCOPY failed
  sync # sync disks
  cp_cmd DETACH $target_vaddr
  echo "Copying disk via FLASHCOPY succeeded ..."
  return $ret_val 
 } # copy_disk()

#+--------------------------------------------------------------------------+
function get_parmfile_info()
# Bring 191 minidisk online to be read by cmsfs and check for two PARMFILEs
#+--------------------------------------------------------------------------+
 {
  # recycle 191 to pick up latest changes
  chccwdev -d 0.0.0191
  chccwdev -e 0.0.0191
  rc=$?
  if [ $rc != 0 ]; then # unable to enable 191 disk
    echo "unable to enable 0.0.0191, rc from chccwdev = $rc"
    exit 5
  fi
  csmdevice=$(cat /proc/dasd/devices | grep 0191 | awk '{print $7}')
  cmsfslst -d /dev/$csmdevice | grep -i $clone_id | grep PARMFILE
  rc=$?
  if [ $rc != 0 ]; then
    echo "Error: $clone_id PARMFILE not found on 191 minidisk. Exiting" 
    exit 6
  fi

# get informaton about target
  export local $(cmsfscat -a -d /dev/$csmdevice $clone_id.parmfile)
  target_hostname=$IP_HOST
  target_IP=$IP_ADDR
  target_DNS=$IP_DNS
  target_GW=$IP_GATEWAY
  target_mask=$IP_NETMASK
  target_MTU=$IP_MTU
  prepare_vaddr $READ_DEVNO
  target_readdev=$new_vaddr
  prepare_vaddr $WRITE_DEVNO
  target_writedev=$new_vaddr
  prepare_vaddr $DATA_DEVNO
  target_datadev=$new_vaddr

# get information about source 
  source_guestID=$(cat /proc/sysinfo | grep "VM00 Name" | awk '{print $3}')
  export local $(cmsfscat -a -d /dev/$csmdevice $source_guestID.parmfile)
  prepare_ipaddr $IP_HOST
  source_hostname=$new_ipaddr
  prepare_ipaddr $IP_ADDR
  source_IP=$new_ipaddr
  prepare_ipaddr $IP_DNS
  source_DNS=$new_ipaddr
  prepare_ipaddr $IP_GATEWAY
  source_GW=$new_ipaddr
  prepare_ipaddr $IP_NETMASK
  source_mask=$new_ipaddr
  source_MTU=$IP_MTU
  prepare_vaddr $READ_DEVNO
  source_readdev=$new_vaddr
  prepare_vaddr $WRITE_DEVNO
  source_writedev=$new_vaddr
  prepare_vaddr $DATA_DEVNO
  source_datadev=$new_vaddr
  source_domain=$(cat /etc/resolv.conf | grep domain | awk '{print $2}')
 }

#+--------------------------------------------------------------------------+
function ask_are_you_sure()
# Ask "Are you sure?" - if not, then exit 
#+--------------------------------------------------------------------------+
 {
  echo ""
  echo "WARNING!!: this will copy 100 and 102 disks to $clone_id 100 and 102"
  echo "New host name will be:  $target_hostname"
  echo "New TCP/IP address will be: $target_IP"
  echo "Other network data is retrieved from $clone_id PARMFILE on 191 disk"
  echo -n "Are you sure you want to overwrite these disks (y/n): "
  read ans
  if [ $ans != "y" ]; then
    exit 7
  fi 
 }

#+--------------------------------------------------------------------------+
function copy_system() 
# copy master image on 100 (root fs) and 102 (swap) to target user ID 
#+--------------------------------------------------------------------------+
 {
  echo "Copying 0102 swap space to $clone_id ..."
  copy_disk 0102 to $clone_id 1102  # copy swap space
  rc=$?
  if [ $rc -gt 1 ]; then # both FLASHCOPY and dasdfmt/dd failed
    echo "Copying disk failed with $rc"
    exit 8
  elif [ $rc = 0 ]; then # FLASHCOPY succeeded
    echo "Sleeping 20 seconds for FLASHCOPY to catch up ..."
    sleep 20
  fi
  echo "Copying 0100 root file system to $clone_id ..."
  copy_disk 0100 to $clone_id 1100 # copy /sles9master to target root file system
  rc=$?
  if [ $rc -gt 1 ]; then # both FLASHCOPY and dasdfmt/dd failed
    echo "Copying disk failed with $rc"
    exit 9
  fi
 }


#+--------------------------------------------------------------------------+
function modify_cloned_image() 
# Mount newly copied system over /mnt/sles9cloned and modify networking info
#   Arg 1: target userid 
#   Arg 2: target minidisk 
# Return code
#   0: success 
#+--------------------------------------------------------------------------+
 {
  target_userid=$1
  target_mdisk=$2
  target_vaddr=$3
  echo "Mounting newly cloned image over $cloned_mnt_pt ..."
  cp_cmd LINK $target_userid $target_mdisk $target_vaddr MR
  rc=$?
  if [ $rc != 0 ]; then # LINK failed
    echo "Fatal error: CP LINK $target_userid $target_mdisk $target_vaddr MR failed with $rc"
    exit 10 
  fi
  if [ ! -d $cloned_mnt_pt ]; then 
    mkdir $cloned_mnt_pt
    rc=$?
    if [ $rc != 0 ]; then
      echo "Fatal Error: mkdir $cloned_mnt_pt failed with $rc"
      exit 10
    fi
  fi
  sleep 1
  chccwdev -e 0.0.$target_vaddr 
  rc=$?
  if [ $rc != 0 ]; then
    echo "Fatal error: chccwdev -e 0.0.$target_vaddr failed with $rc"
    exit 10
  fi
  cloned_DASD=`cat /proc/dasd/devices | grep "$target_vaddr(ECKD)" | awk '{ print $7 }'`
  if [ "$cloned_DASD" = "" ]; then
    cat /proc/dasd/devices
    echo "Fatal error: can't find $target_vaddr(ECKD) in /proc/dasd/devices"
    chccwdev -d 0.0.$target_vaddr
    exit 10
  fi
  cloned_fs="/dev/${cloned_DASD}1"
  echo "Mounting $cloned_fs over $cloned_mnt_pt ..."
  mount $cloned_fs $cloned_mnt_pt
  rc=$?
  if [ $rc != 0 ]; then
    echo "Fatal error: mount $cloned_fs $cloned_mnt_pt failed with $rc"
    exit 10
  fi
  echo "Modifying cloned image under $cloned_mnt_pt ..."
  if [ $verbose = "on" ]; then set -vx; fi
  cat $master_mnt_pt/etc/HOSTNAME | sed \
    -e "s/$source_hostname/$target_hostname/g" > $cloned_mnt_pt/etc/HOSTNAME
  cat $master_mnt_pt/etc/hosts | sed \
    -e "s/$source_IP/$target_IP/g" \
    -e "s/$source_guestID/$clone_id/g" \
    -e "s/$source_domain/$source_domain/g" > $cloned_mnt_pt/etc/hosts
  cat $master_mnt_pt/etc/defaultdomain | sed \
    -e "s/$source_domain/$source_domain/g" > $cloned_mnt_pt/etc/defaultdomain
  cat $master_mnt_pt/etc/sysconfig/network/routes | sed \
    -e "s/$source_GW/$target_GW/g" > $cloned_mnt_pt/etc/sysconfig/network/routes
  cat $master_mnt_pt/etc/sysconfig/network/ifcfg-qeth-bus-ccw-0.0.$source_readdev | sed \
    -e "s/$source_IP/$target_IP/g" \
    -e "s/$source_MTU/$target_MTU/g" \
    -e "s/$source_mask/$target_mask/g" > \
    $cloned_mnt_pt/etc/sysconfig/network/ifcfg-qeth-bus-ccw-0.0.$target_readdev
  cat $master_mnt_pt/etc/resolv.conf | sed \
    -e "s/$source_domain/$source_domain/g"\
    -e "s/$source_DNS/$target_DNS/g" > $cloned_mnt_pt/etc/resolv.conf
  if [ $verbose = "on" ]; then set +vx; fi
# Regenerate SSH keys
  echo "Regenerating SSH keys in $cloned_mnt_pt/etc/ssh/ ..."
  rm $cloned_mnt_pt/etc/ssh/ssh_host*
  ssh-keygen -t rsa -N "" -q -f $cloned_mnt_pt/etc/ssh/ssh_host_rsa_key
  ssh-keygen -t dsa -N "" -q -f $cloned_mnt_pt/etc/ssh/ssh_host_dsa_key
  ssh-keygen -t rsa1 -N "" -q -f $cloned_mnt_pt/etc/ssh/ssh_host_key
# Remove any old entry, then copy clone's public key to known_hosts file
  echo "Adding $target_IP to known_hosts file"
  cd /root/.ssh
  grep -v $target_IP known_hosts > known_hosts.temp
  mv known_hosts.temp known_hosts
# construct known_hosts entry with three fields: IP@ "ssh-rsa" clones_public_key
  new_key=`cat $cloned_mnt_pt/etc/ssh/ssh_host_rsa_key.pub | awk '{print $2}'` 
  echo "$target_IP ssh-rsa $new_key" >> known_hosts
# clean up
  sync # sync disks
  umount $cloned_mnt_pt
  chccwdev -d 0.0.$target_vaddr
  cp_cmd DETACH $target_vaddr 
  return 0
 }

#+--------------------------------------------------------------------------+
function make_backup_dir()
# Create a directory /backup/<VM-ID>-on-<IP.addr> if it doesn't exist 
# e.g. LINUX04 with IP@ 129.40.178.44 would create /backup/LINUX04-on-129.40.178.44/
#+--------------------------------------------------------------------------+
 {
  if [ -d $backup_dir ]; then # main backup directory exists
    echo "Creating a directory under $backup_dir"
    cd $backup_dir
    backup_subdir=$clone_id-on-$target_IP
    if [ -d $backup_subdir ]; then # backup subdirectory exists
      echo "$backup_dir/$backup_subdir/ already exists"
    else
      mkdir $backup_subdir
      if [ ! $? = 0 ]; then # there was an error from mkdir command
        echo "Error creating directory $backup_dir/$backup_subdir/"
      else
        echo "Created directory $backup_dir/$backup_subdir/"
      fi
    fi
  fi
 }

# main() # the first three lines are some important global variables
master_mnt_pt="/sles9master"                # set directory of master root file system 
cloned_mnt_pt="/mnt/sles9cloned"            # set directory of temporary mount point
backup_dir="/backup/linux"                  # set directory of Linux backups

process_arguments $@                        # process arguments passed by user 
check_target_id                             # be sure user ID exists and is logged off
get_parmfile_info                           # get information from source and target parm files
ask_are_you_sure                            # confirm that disks will be overwritten
copy_system                                 # copy 100 and 102 disks to target ID
modify_cloned_image $clone_id 100 1100      # modify newly copied system
cp_cmd XAUTOLOG $clone_id                   # bring new clone to life
make_backup_dir                             # make a backup directory
echo "Successfully cloned $master_mnt_pt to $clone_id"
echo "You should be able to ping $target_IP within one minute"
exit 0
